# coding: utf-8

from __future__ import absolute_import
from .base_model_ import Model
from datetime import date, datetime
from typing import List, Dict
from ..util import deserialize_model


class ItemImage(Model):
  def __init__(self,
               url=None,
               height=None,
               width=None,
               unit=None):
    self.bl_types = {
      'url': str,
      'height': str,
      'width': str,
      'unit': str
    }

    self.attribute_map = {
      'url': 'url',
      'height': 'height',
      'width': 'width',
      'unit': 'unit'
    }

    self._url = url
    self._height = height
    self._width = width
    self._unit = unit

  @classmethod
  def from_dict(cls, dikt) -> 'ItemImage':
    return deserialize_model(dikt, cls)

  @property
  def url(self) -> str:
    return self._url

  @url.setter
  def url(self, url: str):
    self._url = url

  @property
  def height(self) -> str:
    return self._height

  @height.setter
  def height(self, height: str):
    self._height = height

  @property
  def width(self) -> str:
    return self._width

  @width.setter
  def width(self, width: str):
    self._width= width

  @property
  def unit(self) -> str:
    return self._unit

  @unit.setter
  def unit(self, unit: str):
    self._unit = unit



# coding: utf-8

from __future__ import absolute_import
from bs4 import BeautifulSoup
from stylelens_crawl_amazon.model import Item
from stylelens_crawl_amazon.model import ItemImage
from stylelens_crawl_amazon.model import ItemPrice
from stylelens_crawl_amazon.model.item_search_data import ItemSearchData
import traceback


class ItemSearch(object):
  def __init__(self,
               amazon,
               keywords=None,
               search_index=None,
               response_groups=None,
               browse_node=None,
               sort=None,
               item_page=None):

    self._amazon = amazon
    self._keywords = keywords
    self._search_index = search_index
    self._response_groups = response_groups
    self._browse_node = browse_node
    self._sort = sort
    self._item_page = item_page
    self._similar_item_ids = []
    self._items = []
    self._item_search_data = None

  def search(self):
    page = 1
    total_pages = 0
    max_page = 10
    while True:
      print('keyword = ' + self._keywords)
      print('browsenode = ' + self._browse_node)
      print('sort = ' + self._sort)
      print('search_index = ' + self._search_index)
      res = self._amazon.ItemSearch(Keywords=self._keywords,
                              SearchIndex=self._search_index,
                              BrowseNode=self._browse_node,
                              Availability='Available',
                              ItemPage=page,
                              Sort=self._sort,
                              ResponseGroup=self._response_groups)
      soup = BeautifulSoup(res, "xml")

      # print(soup.prettify())
      # self._log(soup.prettify())
      items = soup.find('Items')
      print('page = ' + str(page))

      request = items.find('Request')
      isValid = request.IsValid.text
      if isValid != 'True':
        print("isValid is not True")
        return None, None

      total_pages_tag = items.find('TotalPages')
      if total_pages_tag != None:
        total_pages = int(total_pages_tag.text)
        # print('total pages : ' + total_pages_tag.text)
      else:
        print('')

      if items.Item:
        item = items.Item
      else:
        break

      while True:
        self._extract_item(item)

        item = item.next_sibling
        if not item:
          break

      page = page + 1

      if page > max_page or page > total_pages:
        break

    return self._items, self._similar_item_ids

  @property
  def search_data(self) -> ItemSearchData:
    return self._search_data

  @search_data.setter
  def search_data(self, search_data: ItemSearchData):
    self._search_data = search_data

  def _extract_item(self, data, parent_detail_page=None, parent_add_to_wishlist_link=None):
    item = Item()
    item.asin = data.ASIN.text
    try:
      if data.ParentASIN:
        item.parent_asin = data.ParentASIN.text

      if data.DetailPageURL:
        item.detail_page_link = data.DetailPageURL.text
      elif parent_detail_page != None:
        item.detail_page_link = parent_detail_page.replace(item.parent_asin, item.asin)

      if data.ItemLinks:
        item_link = data.ItemLinks.ItemLink
        while True:
          if item_link.Description:
            if 'Add To Wishlist' == item_link.Description.text:
              item.add_to_wishlist_link = item_link.URL.text
          item_link = item_link.next_sibling
          if not item_link:
            break
      elif parent_add_to_wishlist_link != None:
        item.add_to_wishlist_link = parent_add_to_wishlist_link.replace(item.parent_asin, item.asin)

      self._extract_images(item, data)


      if data.ItemAttributes:
        self._extract_item_attributes(item, data.ItemAttributes)
    except Exception as e:
      traceback.print_exc(limit=None)
      return

    if item.m_image is not None:
      self._items.append(item)

    if data.Variations:
      self._extract_variations(data.Variations, item.detail_page_link, item.add_to_wishlist_link)

    if data.SimilarProducts:
      similar_products = data.SimilarProducts.find_all('SimilarProduct')
      for product in similar_products:
        if product.ASIN:
          self._similar_item_ids.append(product.ASIN.text)

  def _extract_variations(self, variations, parent_detail_page, parent_add_to_wishlist_link):

    if variations.Item:
      item = variations.Item
    else:
      return

    while True:
      self._extract_item(item, parent_detail_page=parent_detail_page, parent_add_to_wishlist_link=parent_add_to_wishlist_link)
      item = item.next_sibling
      if not item:
        break

  def _extract_images(self, item_attributes, data):
    if data.SmallImage:
      ii = ItemImage()
      ii.url = data.SmallImage.URL.text
      ii.height = data.SmallImage.Height.text
      ii.width = data.SmallImage.Width.text
      ii.unit = data.SmallImage.Width['Units']
      item_attributes.s_image = ii

    if data.MediumImage:
      ii = ItemImage()
      ii.url = data.MediumImage.URL.text
      ii.height = data.MediumImage.Height.text
      ii.width = data.MediumImage.Width.text
      ii.unit = data.MediumImage.Width['Units']
      item_attributes.m_image = ii

    if data.LargeImage:
      ii = ItemImage()
      ii.url = data.LargeImage.URL.text
      ii.height = data.LargeImage.Height.text
      ii.width = data.LargeImage.Width.text
      ii.unit = data.LargeImage.Width['Units']
      item_attributes.l_image = ii

  def _extract_item_attributes(self, item, data):
    if data.Binding:
      item.binding = data.Binding.text
    if data.Brand:
      item.brand = data.Brand.text
    if data.Department:
      item.department = data.Department.text
    if data.Color:
      item.color = data.Color.text
    if data.ClothingSize:
      item.clothing_size = data.ClothingSize.text
    if data.ListPrice:
      p = ItemPrice()
      p.amount = data.ListPrice.Amount.text
      p.currency_code = data.ListPrice.CurrencyCode.text
      p.formatted_price = data.ListPrice.FormattedPrice.text

    if data.Feature:
      item.features = []
      feature = data.Feature
      while True:
        item.features.append(feature.text)
        feature = feature.next_sibling
        if feature.name != 'Feature':
          break

    if data.ProductGroup:
      item.product_group = data.ProductGroup.text
    if data.ProductTypeName:
      item.product_type_name = data.ProductTypeName.text
    if data.Title:
      item.title = data.Title.text

  @property
  def item_page(self) -> str:
    return self._item_page

  @item_page.setter
  def item_page(self, item_page: str):
    self._item_page = item_page

  def _log(self, data):
    f = open('search_log.txt', 'a', encoding='UTF-8')
    f.write(str(data))
    f.close()



from __future__ import absolute_import

import os
from stylelens_crawl_amazon.item_search import ItemSearch
from stylelens_crawl_amazon.model.item_search_data import ItemSearchData
from stylelens_crawl_amazon.item_lookup import ItemLookup
from .browse_nodes import BrowseNodes

CATEGORIES     = 'data/categories.txt'
SEARCH_INDICES = 'data/search_indices.txt'
SORTS          = 'data/sorts.txt'
ITEM_SEARCH_RESPONSE_GROUPS= 'data/item_search_response_groups.txt'
ITEM_LOOKUP_RESPONSE_GROUPS= 'data/item_lookup_response_groups.txt'
ATTRIB_TEXTURE = 'data/attribute_texture.txt'
ATTRIB_FABRIC  = 'data/attribute_fabric.txt'
ATTRIB_SHAPE   = 'data/attribute_shape.txt'
ATTRIB_PART    = 'data/attribute_part.txt'
ATTRIB_STYLE   = 'data/attribute_style.txt'

class SearchFactory(object):
  def __init__(self, amazon, generate_item_searches=False):
    self._amazon = amazon
    self._keywords = []
    self._browse_nodes = {}
    self._search_indices = []
    self._categories = []
    self._attrib_list = []
    self._item_searches = []
    self._sorts = []
    self._search_response_groups = []
    self._lookup_response_groups = []
    self._item_ids = {}
    self._similar_item_ids = []

    if generate_item_searches == True:
      self._init()

  def search(self, item_searches):
    count = 0

    for i in item_searches:
      print('search index = ' + str(count))
      items, similar_item_ids = i.search()
      count = count + 1
      self._save_similar_item_ids(similar_item_ids)
      yield items

  def lookup_similar_items(self):
    _item_lookups = self._lookup_similar_items()
    for l in _item_lookups:
      items = l.lookup()
      print(len(items))
      yield items

  def get_item_searches(self):
    return self._item_searches

  def get_similar_items(self):
    return self._similar_item_ids

  def _init(self):
    self._browse_nodes = self._get_browse_nodes()
    if self._browse_nodes is None:
      print('Can not get BrowseNodes from AWS')
      return
    self._init_attributes()
    self._init_categories()
    self._init_search_indices()
    self._init_item_search_response_groups()
    self._init_item_lookup_response_groups()
    self._init_sorts()
    self._generate_keywords_small() # or self._generate_keywords_large()
    self._item_searches = self._generate_item_searches()

  def _init_categories(self):
    self._categories.extend(self._get_items(CATEGORIES))

  def _init_item_search_response_groups(self):
    self._search_response_groups.extend(self._get_items(ITEM_SEARCH_RESPONSE_GROUPS))

  def _init_item_lookup_response_groups(self):
    self._lookup_response_groups.extend(self._get_items(ITEM_LOOKUP_RESPONSE_GROUPS))

  def _init_sorts(self):
    self._sorts.extend(self._get_items(SORTS))

  def _init_search_indices(self):
    self._search_indices.extend(self._get_items(SEARCH_INDICES))

  def _init_attributes(self):
    self._attrib_list.append(self._get_items(ATTRIB_TEXTURE))
    self._attrib_list.append(self._get_items(ATTRIB_FABRIC))
    self._attrib_list.append(self._get_items(ATTRIB_SHAPE))
    self._attrib_list.append(self._get_items(ATTRIB_PART))
    self._attrib_list.append(self._get_items(ATTRIB_STYLE))

  def _get_browse_nodes(self):
    try:
      bn = BrowseNodes()
    except Exception as e:
      print(e)
      return None

    return bn.get_browse_nodes()

  def _generate_keywords_small(self):
    for c in self._categories:
      self._keywords.append(c)

    for attributes in self._attrib_list:
      for a in attributes:
        self._keywords.append(a)

  def _generate_keywords_large(self):
    for c in self._categories:
      self._keywords.append(c)
      for attributes in self._attrib_list:
        for a in attributes:
          self._keywords.append(a + ' ' + c)

  def _generate_item_searches(self):
    for search_index in self._search_indices:
      for sort in self._sorts:
        for keyword in self._keywords:
          response_groups = ','.join(self._search_response_groups)
          if search_index == 'Apparel':
            for browse_node in self._browse_nodes.values():
              yield self._generate_item_search(search_index=search_index,
                                               sort=sort,
                                               keyword=keyword,
                                               response_groups=response_groups,
                                               browse_node=browse_node)
          else:
            yield self._generate_item_search(search_index=search_index,
                                             sort=sort,
                                             keyword=keyword,
                                             response_groups=response_groups)

  def _generate_item_search(self,
                            search_index,
                            sort,
                            keyword,
                            response_groups,
                            browse_node=None):
    i = ItemSearch(amazon=self._amazon)
    data = ItemSearchData()
    data.keywords = keyword
    data.search_index = search_index
    data.browse_node = browse_node
    data.sort = sort
    data.response_groups = response_groups
    i.search_data = data
    return i


  def _get_items(self, file=None):
    this_dir, this_filename = os.path.split(__file__)
    file = os.path.join(this_dir, file)
    items_ = open(file, 'r', encoding='UTF-8')
    items = []
    for i in items_.readlines():
      items.append(i.strip())

    return items

  def _chunks(self, l, n):
    for i in range(0, len(l), n):
      yield l[i:i + n]

  def _lookup_similar_items(self ):
    response_groups = ','.join(self._response_groups)
    for ids in self._chunks(self._similar_item_ids, 10):
      yield self._generate_item_lookup(item_ids=ids,
                                       response_groups=response_groups)


  def _generate_item_lookup(self,
                            item_ids,
                            response_groups=None):
    i = ItemLookup(amazon=self._amazon, item_ids=item_ids)
    i.response_groups = response_groups

    return i

  def _save_similar_item_ids(self, ids):
    self._similar_item_ids.extend(ids)
    self._similar_item_ids = list(set(self._similar_item_ids))




import bottlenose
from .search_factory import SearchFactory


class AwsProductApi(object):
  def __init__(self, generate_item_searches=False):
    self._amazon = bottlenose.Amazon()
    self._search_factory = SearchFactory(self._amazon, generate_item_searches)

  def get_item_searches(self):
    return self._search_factory.get_item_searches()

  def get_similar_items(self):
    return self._search_factory.get_similar_items()

  def item_search(self, item_searches):
    return self._search_factory.search(item_searches)

  def item_lookups(self):
    return self._search_factory.lookup_similar_items()



from __future__ import absolute_import
import os
import bottlenose
from bs4 import BeautifulSoup


BROWSE_NODE_WHITE_LIST_FILE = 'data/browse_nodes_white.txt'
BROWSE_NODE_BLACK_LIST_FILE = 'data/browse_nodes_black.txt'
BROWSE_NODE_CACHE = 'browse_node_cache.txt'

class BrowseNodes(object):
  def __init__(self, name=[], id='7141123011'):
    print('init')
    self._browse_nodes = {}
    self._amazon = bottlenose.Amazon()
    self._white_list = self._get_white_list()
    self._black_list = self._get_black_list()
    self._init_browse_nodes(name, id)

  def get_browse_nodes(self):
    return self._browse_nodes

  def _get_browse_nodes_from_cache(self):
    nodes_ = open(BROWSE_NODE_CACHE, 'r', encoding='UTF-8')
    for n in nodes_.readlines():
      map = n.strip().split(':')
      key = map[0].strip()
      value = map[1].strip()
      self._browse_nodes[key] = value

  def _init_browse_nodes(self, name=None, id=None):
    if os.path.exists(BROWSE_NODE_CACHE):
      self._get_browse_nodes_from_cache()
    else:
      self._get_browse_nodes(name, id)
      self._save_to_cache()

  def _get_browse_nodes(self, name=None, id=None):
    for n in name:
      for b in self._black_list:
        if b in n:
          return None
    if len(name) > 0:
      flattened_name = "/".join(str(x) for x in name)
      for node in self._white_list:
        if node in flattened_name:
          return None

    response = self._amazon.BrowseNodeLookup(BrowseNodeId=id)
    soup = BeautifulSoup(response, "xml")
    isValid = soup.BrowseNodes.Request.IsValid.string

    if isValid == 'True':
      children = soup.BrowseNodes.BrowseNode.Children
      if children is not None:
        browseNodes = soup.BrowseNodes.BrowseNode.Children.find_all('BrowseNode')
        for browseNode in browseNodes:
          name.append(browseNode.Name.string)
          flattened_name = "/".join(str(x) for x in name)
          if flattened_name in self._white_list:
            print(flattened_name + ':' + browseNode.BrowseNodeId.string)
            self._browse_nodes[flattened_name] = browseNode.BrowseNodeId.string
          # print(flattened_name)
          ret = self._get_browse_nodes(name, browseNode.BrowseNodeId.string)
          if ret is None:
            name.pop()
      else:
        return None

  def _get_white_list(self):
    this_dir, this_filename = os.path.split(__file__)
    file = os.path.join(this_dir, BROWSE_NODE_WHITE_LIST_FILE)
    node_list = open(file, 'r', encoding='UTF-8')
    nodes = []
    for node in node_list.readlines():
      nodes.append(node.strip())

    return nodes

  def _get_black_list(self):
    this_dir, this_filename = os.path.split(__file__)
    file = os.path.join(this_dir, BROWSE_NODE_BLACK_LIST_FILE)
    node_list = open(file, 'r', encoding='UTF-8')
    nodes = []
    for node in node_list.readlines():
      nodes.append(node.strip())

    return nodes

  def _save_to_cache(self):
    f = open(BROWSE_NODE_CACHE, 'w', encoding='UTF-8')
    for key in self._browse_nodes.keys():
      data = '{}:{}\n'.format(key, self._browse_nodes[key])
      f.write(data)
    f.close()
